package java.lang;

import java.util.Arrays;
import sun.reflect.annotation.TypeAnnotationParser;
import java.lang.reflect.AnnotatedType;
import java.util.Iterator;
import java.util.LinkedHashMap;
import sun.reflect.annotation.AnnotationParser;
import sun.reflect.annotation.AnnotationSupport;
import java.util.Objects;
import java.lang.annotation.Annotation;
import java.util.HashMap;
import java.util.Collection;
import java.util.HashSet;
import java.util.Set;
import sun.reflect.ConstantPool;
import java.lang.reflect.Executable;
import sun.reflect.generics.factory.CoreReflectionFactory;
import sun.reflect.generics.scope.ClassScope;
import sun.reflect.generics.factory.GenericsFactory;
import java.lang.reflect.Proxy;
import sun.reflect.misc.ReflectUtil;
import java.security.Permissions;
import java.net.URL;
import java.io.InputStream;
import java.lang.reflect.Field;
import java.util.ArrayList;
import sun.reflect.generics.repository.ConstructorRepository;
import java.lang.reflect.Array;
import java.lang.reflect.GenericArrayType;
import sun.reflect.generics.repository.MethodRepository;
import java.lang.reflect.Method;
import java.lang.reflect.InvocationTargetException;
import sun.misc.Unsafe;
import java.security.AccessController;
import java.security.PrivilegedAction;
import sun.security.util.SecurityConstants;
import sun.misc.VM;
import sun.reflect.CallerSensitive;
import sun.reflect.Reflection;
import java.lang.reflect.TypeVariable;
import java.lang.reflect.Modifier;
import sun.reflect.annotation.AnnotationType;
import java.util.Map;
import sun.reflect.ReflectionFactory;
import java.io.ObjectStreamField;
import sun.reflect.generics.repository.ClassRepository;
import java.lang.ref.SoftReference;
import java.security.ProtectionDomain;
import java.lang.reflect.Constructor;
import java.lang.reflect.AnnotatedElement;
import java.lang.reflect.Type;
import java.lang.reflect.GenericDeclaration;
import java.io.Serializable;

public final class Class<T> implements Serializable, GenericDeclaration, Type, AnnotatedElement
{
    private static final int ANNOTATION = 8192;
    private static final int ENUM = 16384;
    private static final int SYNTHETIC = 4096;
    private transient volatile Constructor<T> cachedConstructor;
    private transient volatile Class<?> newInstanceCallerCache;
    private transient String name;
    private final ClassLoader classLoader;
    private static ProtectionDomain allPermDomain;
    private static boolean useCaches;
    private transient volatile SoftReference<Class.ReflectionData<T>> reflectionData;
    private transient volatile int classRedefinedCount;
    private transient volatile ClassRepository genericInfo;
    private static final long serialVersionUID = 3206093459760846163L;
    private static final ObjectStreamField[] serialPersistentFields;
    private static ReflectionFactory reflectionFactory;
    private static boolean initted;
    private transient volatile T[] enumConstants;
    private transient volatile Map<String, T> enumConstantDirectory;
    private transient volatile Class.AnnotationData annotationData;
    private transient volatile AnnotationType annotationType;
    transient ClassValue.ClassValueMap classValueMap;
    
    private static native void registerNatives();
    
    private Class(ClassLoader classLoader) {
        this.classRedefinedCount = 0;
        this.enumConstants = null;
        this.enumConstantDirectory = null;
        this.classLoader = classLoader;
    }
    
    @Override
    public String toString() {
        return (this.isInterface() ? "interface " : (this.isPrimitive() ? "" : "class ")) + this.getName();
    }
    
    public String toGenericString() {
        if (this.isPrimitive()) {
            return this.toString();
        }
        StringBuilder sb = new StringBuilder();
        int mod = this.getModifiers() & Modifier.classModifiers();
        if (mod != 0) {
            sb.append(Modifier.toString(mod));
            sb.append(' ');
        }
        if (this.isAnnotation()) {
            sb.append('@');
        }
        if (this.isInterface()) {
            sb.append("interface");
        }
        else if (this.isEnum()) {
            sb.append("enum");
        }
        else {
            sb.append("class");
        }
        sb.append(' ');
        sb.append(this.getName());
        TypeVariable[] typeParameters = this.getTypeParameters();
        if (typeParameters.length > 0) {
            int n = 1;
            sb.append('<');
            for (int length = typeParameters.length, i = 0; i < length; n = 0, ++i) {
                TypeVariable<Object> typeVariable = typeParameters[i];
                if (n == 0) {
                    sb.append(',');
                }
                sb.append(typeVariable.getTypeName());
            }
            sb.append('>');
        }
        return sb.toString();
    }
    
    @CallerSensitive
    public static Class<?> forName(String s) throws ClassNotFoundException {
        Class<Object> callerClass = Reflection.getCallerClass();
        return forName0(s, true, ClassLoader.getClassLoader(callerClass), callerClass);
    }
    
    @CallerSensitive
    public static Class<?> forName(String s, boolean b, ClassLoader classLoader) throws ClassNotFoundException {
        Class<Object> callerClass = null;
        SecurityManager securityManager = System.getSecurityManager();
        if (securityManager != null) {
            callerClass = Reflection.getCallerClass();
            if (VM.isSystemDomainLoader(classLoader) && !VM.isSystemDomainLoader(ClassLoader.getClassLoader(callerClass))) {
                securityManager.checkPermission(SecurityConstants.GET_CLASSLOADER_PERMISSION);
            }
        }
        return forName0(s, b, classLoader, callerClass);
    }
    
    private static native Class<?> forName0(String p0, boolean p1, ClassLoader p2, Class<?> p3) throws ClassNotFoundException;
    
    @CallerSensitive
    public T newInstance() throws InstantiationException, IllegalAccessException {
        if (System.getSecurityManager() != null) {
            this.checkMemberAccess(0, Reflection.getCallerClass(), false);
        }
        if (this.cachedConstructor == null) {
            if (this == Class.class) {
                throw new IllegalAccessException("Can not call newInstance() on the Class for java.lang.Class");
            }
            try {
                final Constructor<T> constructor0 = this.getConstructor0(new Class[0], 1);
                AccessController.doPrivileged(new PrivilegedAction<Void>() {
                    @Override
                    public Void run() {
                        constructor0.setAccessible(true);
                        return null;
                    }
                });
                this.cachedConstructor = constructor0;
            }
            catch (NoSuchMethodException ex) {
                throw (InstantiationException)new InstantiationException(this.getName()).initCause(ex);
            }
        }
        Constructor<Object> cachedConstructor = this.cachedConstructor;
        int modifiers = cachedConstructor.getModifiers();
        if (!Reflection.quickCheckMemberAccess(this, modifiers)) {
            Class<Object> callerClass = Reflection.getCallerClass();
            if (this.newInstanceCallerCache != callerClass) {
                Reflection.ensureMemberAccess(callerClass, this, null, modifiers);
                this.newInstanceCallerCache = callerClass;
            }
        }
        try {
            return cachedConstructor.newInstance((Object[])null);
        }
        catch (InvocationTargetException ex2) {
            Unsafe.getUnsafe().throwException(ex2.getTargetException());
            return null;
        }
    }
    
    public native boolean isInstance(Object p0);
    
    public native boolean isAssignableFrom(Class<?> p0);
    
    public native boolean isInterface();
    
    public native boolean isArray();
    
    public native boolean isPrimitive();
    
    public boolean isAnnotation() {
        return (this.getModifiers() & 0x2000) != 0x0;
    }
    
    public boolean isSynthetic() {
        return (this.getModifiers() & 0x1000) != 0x0;
    }
    
    public String getName() {
        String name = this.name;
        if (name == null) {
            name = (this.name = this.getName0());
            return name;
        }
        return name;
    }
    
    private native String getName0();
    
    @CallerSensitive
    public ClassLoader getClassLoader() {
        ClassLoader classLoader0 = this.getClassLoader0();
        if (classLoader0 == null) {
            return null;
        }
        if (System.getSecurityManager() != null) {
            ClassLoader.checkClassLoaderPermission(classLoader0, Reflection.getCallerClass());
            return classLoader0;
        }
        return classLoader0;
    }
    
    ClassLoader getClassLoader0() {
        return this.classLoader;
    }
    
    @Override
    public TypeVariable<Class<T>>[] getTypeParameters() {
        ClassRepository genericInfo = this.getGenericInfo();
        if (genericInfo != null) {
            return (TypeVariable[])genericInfo.getTypeParameters();
        }
        return (TypeVariable[])new TypeVariable[0];
    }
    
    public native Class<? super T> getSuperclass();
    
    public Type getGenericSuperclass() {
        ClassRepository genericInfo = this.getGenericInfo();
        if (genericInfo == null) {
            return this.getSuperclass();
        }
        if (this.isInterface()) {
            return null;
        }
        return genericInfo.getSuperclass();
    }
    
    public Package getPackage() {
        return Package.getPackage(this);
    }
    
    public Class<?>[] getInterfaces() {
        Class.ReflectionData<T> reflectionData = this.reflectionData();
        if (reflectionData == null) {
            return this.getInterfaces0();
        }
        Class[] interfaces = reflectionData.interfaces;
        if (interfaces == null) {
            interfaces = this.getInterfaces0();
            reflectionData.interfaces = interfaces;
        }
        return (Class<?>[])interfaces.clone();
    }
    
    private native Class<?>[] getInterfaces0();
    
    public Type[] getGenericInterfaces() {
        ClassRepository genericInfo = this.getGenericInfo();
        return (genericInfo == null) ? this.getInterfaces() : genericInfo.getSuperInterfaces();
    }
    
    public native Class<?> getComponentType();
    
    public native int getModifiers();
    
    public native Object[] getSigners();
    
    native void setSigners(Object[] p0);
    
    @CallerSensitive
    public Method getEnclosingMethod() throws SecurityException {
        Class.EnclosingMethodInfo enclosingMethodInfo = this.getEnclosingMethodInfo();
        if (enclosingMethodInfo == null) {
            return null;
        }
        if (!enclosingMethodInfo.isMethod()) {
            return null;
        }
        MethodRepository make = MethodRepository.make(enclosingMethodInfo.getDescriptor(), this.getFactory());
        Class<Object> class1 = toClass(make.getReturnType());
        Type[] parameterTypes = make.getParameterTypes();
        Class[] array = new Class[parameterTypes.length];
        for (int i = 0; i < array.length; ++i) {
            array[i] = toClass(parameterTypes[i]);
        }
        Class<Object> enclosingClass = enclosingMethodInfo.getEnclosingClass();
        enclosingClass.checkMemberAccess(1, Reflection.getCallerClass(), true);
        for (Method method : enclosingClass.getDeclaredMethods()) {
            if (method.getName().equals(enclosingMethodInfo.getName())) {
                Class[] parameterTypes2 = method.getParameterTypes();
                if (parameterTypes2.length == array.length) {
                    boolean n = true;
                    int k = 0;
                    while (k < parameterTypes2.length) {
                        if (!parameterTypes2[k].equals(array[k])) {
                            n = false;
                            break;
                        }
                        ++k;
                    }
                    if (n && method.getReturnType().equals(class1)) {
                        return method;
                    }
                }
            }
        }
        throw new InternalError("Enclosing method not found");
    }
    
    private native Object[] getEnclosingMethod0();
    
    private Class.EnclosingMethodInfo getEnclosingMethodInfo() {
        Object[] enclosingMethod0 = this.getEnclosingMethod0();
        if (enclosingMethod0 == null) {
            return null;
        }
        return new EnclosingMethodInfo(enclosingMethod0);
    }
    
    private static Class<?> toClass(Type type) {
        if ((type instanceof GenericArrayType)) {
            return Array.newInstance(toClass(((GenericArrayType)type).getGenericComponentType()), 0).getClass();
        }
        return (Class)type;
    }
    
    @CallerSensitive
    public Constructor<?> getEnclosingConstructor() throws SecurityException {
        Class.EnclosingMethodInfo enclosingMethodInfo = this.getEnclosingMethodInfo();
        if (enclosingMethodInfo == null) {
            return null;
        }
        if (!enclosingMethodInfo.isConstructor()) {
            return null;
        }
        Type[] parameterTypes = ConstructorRepository.make(enclosingMethodInfo.getDescriptor(), this.getFactory()).getParameterTypes();
        Class[] array = new Class[parameterTypes.length];
        for (int i = 0; i < array.length; ++i) {
            array[i] = toClass(parameterTypes[i]);
        }
        Class<Object> enclosingClass = enclosingMethodInfo.getEnclosingClass();
        enclosingClass.checkMemberAccess(1, Reflection.getCallerClass(), true);
        for (Constructor constructor : enclosingClass.getDeclaredConstructors()) {
            Class[] parameterTypes2 = constructor.getParameterTypes();
            if (parameterTypes2.length == array.length) {
                boolean n = true;
                int k = 0;
                while (k < parameterTypes2.length) {
                    if (!parameterTypes2[k].equals(array[k])) {
                        n = false;
                        break;
                    }
                    ++k;
                }
                if (!(n == 0)) {
                    return constructor;
                }
            }
        }
        throw new InternalError("Enclosing constructor not found");
    }
    
    @CallerSensitive
    public Class<?> getDeclaringClass() throws SecurityException {
        Class<Object> declaringClass0 = this.getDeclaringClass0();
        if (declaringClass0 != null) {
            declaringClass0.checkPackageAccess(ClassLoader.getClassLoader(Reflection.getCallerClass()), true);
            return declaringClass0;
        }
        return declaringClass0;
    }
    
    private native Class<?> getDeclaringClass0();
    
    @CallerSensitive
    public Class<?> getEnclosingClass() throws SecurityException {
        Class.EnclosingMethodInfo enclosingMethodInfo = this.getEnclosingMethodInfo();
        Class<Object> declaringClass;
        if (enclosingMethodInfo == null) {
            declaringClass = this.getDeclaringClass();
        }
        else {
            Class<Object> enclosingClass = enclosingMethodInfo.getEnclosingClass();
            if (enclosingClass == this || enclosingClass == null) {
                throw new InternalError("Malformed enclosing method information");
            }
            declaringClass = enclosingClass;
        }
        if (declaringClass != null) {
            declaringClass.checkPackageAccess(ClassLoader.getClassLoader(Reflection.getCallerClass()), true);
            return declaringClass;
        }
        return declaringClass;
    }
    
    public String getSimpleName() {
        if (this.isArray()) {
            return this.getComponentType().getSimpleName() + "[]";
        }
        String simpleBinaryName = this.getSimpleBinaryName();
        if (simpleBinaryName == null) {
            String name = this.getName();
            return name.substring(name.lastIndexOf(".") + 1);
        }
        int length = simpleBinaryName.length();
        if (length < 1 || simpleBinaryName.charAt(0) != '$') {
            throw new InternalError("Malformed class name");
        }
        int n;
        for (n = 1; n < length && isAsciiDigit(simpleBinaryName.charAt(n)); ++n) {}
        return simpleBinaryName.substring(n);
    }
    
    @Override
    public String getTypeName() {
        if (this.isArray()) {
            try {
                int n = 0;
                while (this.isArray()) {
                    ++n;
                    this = this.getComponentType();
                }
                StringBuilder o = new StringBuilder();
                o.append(this.getName());
                for (int i = 0; i < n; ++i) {
                    o.append("[]");
                }
                return o.toString();
            }
            catch (Throwable t) {}
        }
        return this.getName();
    }
    
    private static boolean isAsciiDigit(char c) {
        return '0' <= c && c <= '9';
    }
    
    public String getCanonicalName() {
        if (this.isArray()) {
            String canonicalName2 = this.getComponentType().getCanonicalName();
            if (canonicalName2 != null) {
                return canonicalName2 + "[]";
            }
            return null;
        }
        else {
            if (this.isLocalOrAnonymousClass()) {
                return null;
            }
            Class<Object> enclosingClass = this.getEnclosingClass();
            if (enclosingClass == null) {
                return this.getName();
            }
            String canonicalName = enclosingClass.getCanonicalName();
            if (canonicalName == null) {
                return null;
            }
            return canonicalName + "." + this.getSimpleName();
        }
    }
    
    public boolean isAnonymousClass() {
        return "".equals(this.getSimpleName());
    }
    
    public boolean isLocalClass() {
        return this.isLocalOrAnonymousClass() && !this.isAnonymousClass();
    }
    
    public boolean isMemberClass() {
        return this.getSimpleBinaryName() != null && !this.isLocalOrAnonymousClass();
    }
    
    private String getSimpleBinaryName() {
        Class<Object> enclosingClass = this.getEnclosingClass();
        if (enclosingClass == null) {
            return null;
        }
        try {
            return this.getName().substring(enclosingClass.getName().length());
        }
        catch (IndexOutOfBoundsException ex) {
            throw new InternalError("Malformed class name", ex);
        }
    }
    
    private boolean isLocalOrAnonymousClass() {
        return this.getEnclosingMethodInfo() != null;
    }
    
    @CallerSensitive
    public Class<?>[] getClasses() {
        this.checkMemberAccess(0, Reflection.getCallerClass(), false);
        return (Class[])AccessController.doPrivileged(new PrivilegedAction<Class<?>[]>() {
            @Override
            public Class<?>[] run() {
                ArrayList<Object> list = new ArrayList();
                for (Class clazz = Class.this; clazz != null; clazz = clazz.getSuperclass()) {
                    Class[] declaredClasses = clazz.getDeclaredClasses();
                    for (int i = 0; i < declaredClasses.length; ++i) {
                        if (Modifier.isPublic(declaredClasses[i].getModifiers())) {
                            list.add(declaredClasses[i]);
                        }
                    }
                }
                return (Class[])list.toArray(new Class[0]);
            }
        });
    }
    
    @CallerSensitive
    public Field[] getFields() throws SecurityException {
        this.checkMemberAccess(0, Reflection.getCallerClass(), true);
        return copyFields(this.privateGetPublicFields(null));
    }
    
    @CallerSensitive
    public Method[] getMethods() throws SecurityException {
        this.checkMemberAccess(0, Reflection.getCallerClass(), true);
        return copyMethods(this.privateGetPublicMethods());
    }
    
    @CallerSensitive
    public Constructor<?>[] getConstructors() throws SecurityException {
        this.checkMemberAccess(0, Reflection.getCallerClass(), true);
        return copyConstructors(this.privateGetDeclaredConstructors(true));
    }
    
    @CallerSensitive
    public Field getField(String s) throws NoSuchFieldException, SecurityException {
        this.checkMemberAccess(0, Reflection.getCallerClass(), true);
        Field field0 = this.getField0(s);
        if (field0 == null) {
            throw new NoSuchFieldException(s);
        }
        return field0;
    }
    
    @CallerSensitive
    public Method getMethod(String s, Class<?>... array) throws NoSuchMethodException, SecurityException {
        this.checkMemberAccess(0, Reflection.getCallerClass(), true);
        Method method0 = this.getMethod0(s, array, true);
        if (method0 == null) {
            throw new NoSuchMethodException(this.getName() + "." + s + argumentTypesToString(array));
        }
        return method0;
    }
    
    @CallerSensitive
    public Constructor<T> getConstructor(Class<?>... array) throws NoSuchMethodException, SecurityException {
        this.checkMemberAccess(0, Reflection.getCallerClass(), true);
        return this.getConstructor0(array, 0);
    }
    
    @CallerSensitive
    public Class<?>[] getDeclaredClasses() throws SecurityException {
        this.checkMemberAccess(1, Reflection.getCallerClass(), false);
        return this.getDeclaredClasses0();
    }
    
    @CallerSensitive
    public Field[] getDeclaredFields() throws SecurityException {
        this.checkMemberAccess(1, Reflection.getCallerClass(), true);
        return copyFields(this.privateGetDeclaredFields(false));
    }
    
    @CallerSensitive
    public Method[] getDeclaredMethods() throws SecurityException {
        this.checkMemberAccess(1, Reflection.getCallerClass(), true);
        return copyMethods(this.privateGetDeclaredMethods(false));
    }
    
    @CallerSensitive
    public Constructor<?>[] getDeclaredConstructors() throws SecurityException {
        this.checkMemberAccess(1, Reflection.getCallerClass(), true);
        return copyConstructors(this.privateGetDeclaredConstructors(false));
    }
    
    @CallerSensitive
    public Field getDeclaredField(String s) throws NoSuchFieldException, SecurityException {
        this.checkMemberAccess(1, Reflection.getCallerClass(), true);
        Field searchFields = searchFields(this.privateGetDeclaredFields(false), s);
        if (searchFields == null) {
            throw new NoSuchFieldException(s);
        }
        return searchFields;
    }
    
    @CallerSensitive
    public Method getDeclaredMethod(String s, Class<?>... array) throws NoSuchMethodException, SecurityException {
        this.checkMemberAccess(1, Reflection.getCallerClass(), true);
        Method searchMethods = searchMethods(this.privateGetDeclaredMethods(false), s, array);
        if (searchMethods == null) {
            throw new NoSuchMethodException(this.getName() + "." + s + argumentTypesToString(array));
        }
        return searchMethods;
    }
    
    @CallerSensitive
    public Constructor<T> getDeclaredConstructor(Class<?>... array) throws NoSuchMethodException, SecurityException {
        this.checkMemberAccess(1, Reflection.getCallerClass(), true);
        return this.getConstructor0(array, 1);
    }
    
    public InputStream getResourceAsStream(String resolveName) {
        resolveName = this.resolveName(resolveName);
        ClassLoader classLoader0 = this.getClassLoader0();
        if (classLoader0 == null) {
            return ClassLoader.getSystemResourceAsStream(resolveName);
        }
        return classLoader0.getResourceAsStream(resolveName);
    }
    
    public URL getResource(String resolveName) {
        resolveName = this.resolveName(resolveName);
        ClassLoader classLoader0 = this.getClassLoader0();
        if (classLoader0 == null) {
            return ClassLoader.getSystemResource(resolveName);
        }
        return classLoader0.getResource(resolveName);
    }
    
    public ProtectionDomain getProtectionDomain() {
        SecurityManager securityManager = System.getSecurityManager();
        if (securityManager != null) {
            securityManager.checkPermission(SecurityConstants.GET_PD_PERMISSION);
        }
        ProtectionDomain protectionDomain = this.getProtectionDomain0();
        if (protectionDomain == null) {
            if (Class.allPermDomain == null) {
                Permissions permissions = new Permissions();
                permissions.add(SecurityConstants.ALL_PERMISSION);
                Class.allPermDomain = new ProtectionDomain(null, permissions);
            }
            protectionDomain = Class.allPermDomain;
            return protectionDomain;
        }
        else {
            return protectionDomain;
        }
    }
    
    private native ProtectionDomain getProtectionDomain0();
    
    static native Class<?> getPrimitiveClass(String p0);
    
    private void checkMemberAccess(int n, Class<?> clazz, boolean b) {
        SecurityManager securityManager = System.getSecurityManager();
        if (securityManager != null) {
            ClassLoader classLoader = ClassLoader.getClassLoader(clazz);
            ClassLoader classLoader2 = this.getClassLoader0();
            if (n != 0 && classLoader != classLoader2) {
                securityManager.checkPermission(SecurityConstants.CHECK_MEMBER_ACCESS_PERMISSION);
            }
            this.checkPackageAccess(classLoader, b);
        }
    }
    
    private void checkPackageAccess(ClassLoader classLoader, boolean b) {
        SecurityManager securityManager = System.getSecurityManager();
        if (securityManager != null) {
            if (ReflectUtil.needsPackageAccessCheck(classLoader, this.getClassLoader0())) {
                String name = this.getName();
                int lastIndex = name.lastIndexOf(46);
                if (lastIndex != -1) {
                    String substring = name.substring(0, lastIndex);
                    if (!Proxy.isProxyClass(this) || ReflectUtil.isNonPublicProxyClass(this)) {
                        securityManager.checkPackageAccess(substring);
                    }
                }
            }
            if (b && Proxy.isProxyClass(this)) {
                ReflectUtil.checkProxyPackageAccess(classLoader, this.getInterfaces());
            }
        }
    }
    
    private String resolveName(String s) {
        if (s == null) {
            return s;
        }
        if (!s.startsWith("/")) {
            while (this.isArray()) {
                this = this.getComponentType();
            }
            String name = this.getName();
            int lastIndex = name.lastIndexOf(46);
            if (lastIndex != -1) {
                s = name.substring(0, lastIndex).replace('.', '/') + "/" + s;
            }
            return s;
        }
        else {
            s = s.substring(1);
            return s;
        }
    }
    
    private Class.ReflectionData<T> reflectionData() {
        SoftReference<Class.ReflectionData<T>> reflectionData = this.reflectionData;
        int classRedefinedCount = this.classRedefinedCount;
        Class.ReflectionData<Object> reflectionData2;
        if (Class.useCaches && reflectionData != null && (reflectionData2 = (Class.ReflectionData)reflectionData.get()) != null && reflectionData2.redefinedCount == classRedefinedCount) {
            return reflectionData2;
        }
        return this.newReflectionData(reflectionData, classRedefinedCount);
    }
    
    private Class.ReflectionData<T> newReflectionData(SoftReference<Class.ReflectionData<T>> reflectionData, int classRedefinedCount) {
        if (!Class.useCaches) {
            return null;
        }
        while (true) {
            Class.ReflectionData referent = new ReflectionData(classRedefinedCount);
            if (Class.Atomic.casReflectionData(this, reflectionData, new SoftReference(referent))) {
                return referent;
            }
            reflectionData = this.reflectionData;
            classRedefinedCount = this.classRedefinedCount;
            Class.ReflectionData<Object> reflectionData2;
            if (reflectionData != null && (reflectionData2 = (Class.ReflectionData)reflectionData.get()) != null && reflectionData2.redefinedCount == classRedefinedCount) {
                return reflectionData2;
            }
            continue;
        }
    }
    
    private native String getGenericSignature0();
    
    private GenericsFactory getFactory() {
        return CoreReflectionFactory.make(this, ClassScope.make(this));
    }
    
    private ClassRepository getGenericInfo() {
        ClassRepository genericInfo = this.genericInfo;
        if (genericInfo == null) {
            String genericSignature0 = this.getGenericSignature0();
            if (genericSignature0 == null) {
                genericInfo = ClassRepository.NONE;
            }
            else {
                genericInfo = ClassRepository.make(genericSignature0, this.getFactory());
            }
            this.genericInfo = genericInfo;
        }
        return (genericInfo != ClassRepository.NONE) ? genericInfo : null;
    }
    
    native byte[] getRawAnnotations();
    
    native byte[] getRawTypeAnnotations();
    
    static byte[] getExecutableTypeAnnotationBytes(Executable executable) {
        return getReflectionFactory().getExecutableTypeAnnotationBytes(executable);
    }
    
    native ConstantPool getConstantPool();
    
    private Field[] privateGetDeclaredFields(boolean b) {
        checkInitted();
        Class.ReflectionData<T> reflectionData = this.reflectionData();
        if (reflectionData != null) {
            Field[] array = b ? reflectionData.declaredPublicFields : reflectionData.declaredFields;
            if (array != null) {
                return array;
            }
        }
        Field[] filterFields = Reflection.filterFields(this, this.getDeclaredFields0(b));
        if (reflectionData != null) {
            if (b) {
                reflectionData.declaredPublicFields = filterFields;
            }
            else {
                reflectionData.declaredFields = filterFields;
            }
        }
        return filterFields;
    }
    
    private Field[] privateGetPublicFields(Set<Class<?>> set) {
        checkInitted();
        Class.ReflectionData<T> reflectionData = this.reflectionData();
        if (reflectionData != null) {
            Field[] publicFields = reflectionData.publicFields;
            if (publicFields != null) {
                return publicFields;
            }
        }
        ArrayList o = new ArrayList();
        if (set == null) {
            set = new HashSet();
        }
        addAll(o, this.privateGetDeclaredFields(true));
        for (Class clazz : this.getInterfaces()) {
            if (!set.contains(clazz)) {
                set.add(clazz);
                addAll(o, clazz.privateGetPublicFields(set));
            }
        }
        if (!this.isInterface()) {
            Class<Object> superclass = this.getSuperclass();
            if (superclass != null) {
                addAll(o, superclass.privateGetPublicFields(set));
            }
        }
        Field[] publicFields2 = new Field[o.size()];
        o.toArray(publicFields2);
        if (reflectionData != null) {
            reflectionData.publicFields = publicFields2;
            return publicFields2;
        }
        return publicFields2;
    }
    
    private static void addAll(Collection<Field> collection, Field[] array) {
        for (int i = 0; i < array.length; ++i) {
            collection.add(array[i]);
        }
    }
    
    private Constructor<T>[] privateGetDeclaredConstructors(boolean b) {
        checkInitted();
        Class.ReflectionData<T> reflectionData = this.reflectionData();
        if (reflectionData != null) {
            Constructor[] array = b ? reflectionData.publicConstructors : reflectionData.declaredConstructors;
            if (array != null) {
                return array;
            }
        }
        Constructor[] declaredConstructors0;
        if (this.isInterface()) {
            declaredConstructors0 = (Constructor[])new Constructor[0];
        }
        else {
            declaredConstructors0 = this.getDeclaredConstructors0(b);
        }
        if (reflectionData != null) {
            if (b) {
                reflectionData.publicConstructors = declaredConstructors0;
            }
            else {
                reflectionData.declaredConstructors = declaredConstructors0;
            }
        }
        return declaredConstructors0;
    }
    
    private Method[] privateGetDeclaredMethods(boolean b) {
        checkInitted();
        Class.ReflectionData<T> reflectionData = this.reflectionData();
        if (reflectionData != null) {
            Method[] array = b ? reflectionData.declaredPublicMethods : reflectionData.declaredMethods;
            if (array != null) {
                return array;
            }
        }
        Method[] filterMethods = Reflection.filterMethods(this, this.getDeclaredMethods0(b));
        if (reflectionData != null) {
            if (b) {
                reflectionData.declaredPublicMethods = filterMethods;
            }
            else {
                reflectionData.declaredMethods = filterMethods;
            }
        }
        return filterMethods;
    }
    
    private Method[] privateGetPublicMethods() {
        checkInitted();
        Class.ReflectionData<T> reflectionData = this.reflectionData();
        if (reflectionData != null) {
            Method[] publicMethods = reflectionData.publicMethods;
            if (publicMethods != null) {
                return publicMethods;
            }
        }
        Class.MethodArray o = new Class.MethodArray();
        o.addAll(this.privateGetDeclaredMethods(true));
        Class.MethodArray o2 = new Class.MethodArray();
        Class[] interfaces = this.getInterfaces();
        for (int length = interfaces.length, i = 0; i < length; ++i) {
            o2.addInterfaceMethods(interfaces[i].privateGetPublicMethods());
        }
        if (!this.isInterface()) {
            Class<Object> superclass = this.getSuperclass();
            if (superclass != null) {
                Class.MethodArray o3 = new Class.MethodArray();
                o3.addAll(superclass.privateGetPublicMethods());
                for (int j = 0; j < o3.length(); ++j) {
                    Method value = o3.get(j);
                    if (value != null && !Modifier.isAbstract(value.getModifiers()) && !value.isDefault()) {
                        o2.removeByNameAndDescriptor(value);
                    }
                }
                o3.addAll(o2);
                o2 = o3;
            }
        }
        for (int k = 0; k < o.length(); ++k) {
            o2.removeByNameAndDescriptor(o.get(k));
        }
        o.addAllIfNotPresent(o2);
        o.removeLessSpecifics();
        o.compactAndTrim();
        Method[] array = o.getArray();
        if (reflectionData != null) {
            reflectionData.publicMethods = array;
            return array;
        }
        return array;
    }
    
    private static Field searchFields(Field[] array, String s) {
        String intern = s.intern();
        int i = 0;
        while (i < array.length) {
            if (array[i].getName() == intern) {
                return getReflectionFactory().copyField(array[i]);
            }
            ++i;
        }
        return null;
    }
    
    private Field getField0(String s) throws NoSuchFieldException {
        Field searchFields;
        if ((searchFields = searchFields(this.privateGetDeclaredFields(true), s)) != null) {
            return searchFields;
        }
        Class[] interfaces = this.getInterfaces();
        int i = 0;
        while (i < interfaces.length) {
            Field field0;
            if ((field0 = interfaces[i].getField0(s)) != null) {
                return field0;
            }
            ++i;
        }
        if (!this.isInterface()) {
            Class<Object> superclass = this.getSuperclass();
            Field field2;
            if (superclass != null && (field2 = superclass.getField0(s)) != null) {
                return field2;
            }
        }
        return null;
    }
    
    private static Method searchMethods(Method[] array, String s, Class<?>[] array2) {
        Method method = null;
        String intern = s.intern();
        for (int i = 0; i < array.length; ++i) {
            Method method2 = array[i];
            if (method2.getName() == intern && arrayContentsEq(array2, method2.getParameterTypes()) && (method == null || method.getReturnType().isAssignableFrom(method2.getReturnType()))) {
                method = method2;
            }
        }
        return (method == null) ? method : getReflectionFactory().copyMethod(method);
    }
    
    private Method getMethod0(String s, Class<?>[] array, boolean b) {
        Class.MethodArray methodArray = new Class.MethodArray(2);
        Method privateGetMethodRecursive = this.privateGetMethodRecursive(s, array, b, methodArray);
        if (privateGetMethodRecursive != null) {
            return privateGetMethodRecursive;
        }
        methodArray.removeLessSpecifics();
        return methodArray.getFirst();
    }
    
    private Method privateGetMethodRecursive(String s, Class<?>[] array, boolean b, Class.MethodArray methodArray) {
        Method searchMethods;
        if ((searchMethods = searchMethods(this.privateGetDeclaredMethods(true), s, array)) != null && (b || !Modifier.isStatic(searchMethods.getModifiers()))) {
            return searchMethods;
        }
        if (!this.isInterface()) {
            Class<Object> superclass = this.getSuperclass();
            Method method0;
            if (superclass != null && (method0 = superclass.getMethod0(s, array, true)) != null) {
                return method0;
            }
        }
        Class[] interfaces = this.getInterfaces();
        for (int length = interfaces.length, i = 0; i < length; ++i) {
            Method method2;
            if ((method2 = interfaces[i].getMethod0(s, array, false)) != null) {
                methodArray.add(method2);
            }
        }
        return null;
    }
    
    private Constructor<T> getConstructor0(Class<?>[] array, int n) throws NoSuchMethodException {
        Constructor[] privateGetDeclaredConstructors = this.privateGetDeclaredConstructors(n == 0);
        int length = privateGetDeclaredConstructors.length;
        int i = 0;
        while (i < length) {
            Constructor<Object> constructor = privateGetDeclaredConstructors[i];
            if (arrayContentsEq(array, constructor.getParameterTypes())) {
                return getReflectionFactory().copyConstructor(constructor);
            }
            ++i;
        }
        throw new NoSuchMethodException(this.getName() + ".<init>" + argumentTypesToString(array));
    }
    
    private static boolean arrayContentsEq(Object[] array, Object[] array2) {
        if (array == null) {
            return array2 == null || array2.length == 0;
        }
        if (array2 == null) {
            return array.length == 0;
        }
        if (array.length != array2.length) {
            return false;
        }
        int i = 0;
        while (i < array.length) {
            if (array[i] != array2[i]) {
                return false;
            }
            ++i;
        }
        return true;
    }
    
    private static Field[] copyFields(Field[] array) {
        Field[] array2 = new Field[array.length];
        ReflectionFactory reflectionFactory = getReflectionFactory();
        for (int i = 0; i < array.length; ++i) {
            array2[i] = reflectionFactory.copyField(array[i]);
        }
        return array2;
    }
    
    private static Method[] copyMethods(Method[] array) {
        Method[] array2 = new Method[array.length];
        ReflectionFactory reflectionFactory = getReflectionFactory();
        for (int i = 0; i < array.length; ++i) {
            array2[i] = reflectionFactory.copyMethod(array[i]);
        }
        return array2;
    }
    
    private static <U> Constructor<U>[] copyConstructors(Constructor<U>[] array) {
        Constructor[] array2 = (Constructor<U>[])array.clone();
        ReflectionFactory reflectionFactory = getReflectionFactory();
        for (int i = 0; i < array2.length; ++i) {
            array2[i] = reflectionFactory.copyConstructor(array2[i]);
        }
        return array2;
    }
    
    private native Field[] getDeclaredFields0(boolean p0);
    
    private native Method[] getDeclaredMethods0(boolean p0);
    
    private native Constructor<T>[] getDeclaredConstructors0(boolean p0);
    
    private native Class<?>[] getDeclaredClasses0();
    
    private static String argumentTypesToString(Class<?>[] array) {
        StringBuilder sb = new StringBuilder();
        sb.append("(");
        if (array != null) {
            for (int i = 0; i < array.length; ++i) {
                if (i > 0) {
                    sb.append(", ");
                }
                Class<?> clazz = array[i];
                sb.append((clazz == null) ? "null" : clazz.getName());
            }
        }
        sb.append(")");
        return sb.toString();
    }
    
    public boolean desiredAssertionStatus() {
        ClassLoader classLoader = this.getClassLoader();
        if (classLoader == null) {
            return desiredAssertionStatus0(this);
        }
        synchronized (classLoader.assertionLock) {
            if (classLoader.classAssertionStatus != null) {
                return classLoader.desiredAssertionStatus(this.getName());
            }
        }
        return desiredAssertionStatus0(this);
    }
    
    private static native boolean desiredAssertionStatus0(Class<?> p0);
    
    public boolean isEnum() {
        return (this.getModifiers() & 0x4000) != 0x0 && this.getSuperclass() == Enum.class;
    }
    
    private static ReflectionFactory getReflectionFactory() {
        if (Class.reflectionFactory == null) {
            Class.reflectionFactory = (ReflectionFactory)AccessController.doPrivileged(new ReflectionFactory.GetReflectionFactoryAction());
        }
        return Class.reflectionFactory;
    }
    
    private static void checkInitted() {
        if (Class.initted) {
            return;
        }
        AccessController.doPrivileged(new PrivilegedAction<Void>() {
            @Override
            public Void run() {
                if (System.out == null) {
                    return null;
                }
                String property = System.getProperty("sun.reflect.noCaches");
                if (property != null && property.equals("true")) {
                    Class.useCaches = false;
                }
                Class.initted = true;
                return null;
            }
        });
        return;
    }
    
    public T[] getEnumConstants() {
        Object[] enumConstantsShared = this.getEnumConstantsShared();
        return (enumConstantsShared != null) ? ((Object[])enumConstantsShared.clone()) : null;
    }
    
    T[] getEnumConstantsShared() {
        if (this.enumConstants == null) {
            if (!this.isEnum()) {
                return null;
            }
            try {
                final Method method = this.getMethod("values", new Class[0]);
                AccessController.doPrivileged(new PrivilegedAction<Void>() {
                    @Override
                    public Void run() {
                        method.setAccessible(true);
                        return null;
                    }
                });
                this.enumConstants = (Object[])(Object[])method.invoke(null, new Object[0]);
            }
            catch (InvocationTargetException | NoSuchMethodException | IllegalAccessException ex) {
                return null;
            }
        }
        return this.enumConstants;
    }
    
    Map<String, T> enumConstantDirectory() {
        if (this.enumConstantDirectory == null) {
            Object[] enumConstantsShared = this.getEnumConstantsShared();
            if (enumConstantsShared == null) {
                throw new IllegalArgumentException(this.getName() + " is not an enum type");
            }
            HashMap<String, Object> enumConstantDirectory = new HashMap(2 * enumConstantsShared.length);
            for (Object o : enumConstantsShared) {
                enumConstantDirectory.put(((Enum)o).name(), o);
            }
            this.enumConstantDirectory = enumConstantDirectory;
        }
        return this.enumConstantDirectory;
    }
    
    public T cast(Object o) {
        if (o != null && !this.isInstance(o)) {
            throw new ClassCastException(this.cannotCastMsg(o));
        }
        return o;
    }
    
    private String cannotCastMsg(Object o) {
        return "Cannot cast " + o.getClass().getName() + " to " + this.getName();
    }
    
    public <U> Class<? extends U> asSubclass(Class<U> clazz) {
        if (clazz.isAssignableFrom(this)) {
            return this;
        }
        throw new ClassCastException(this.toString());
    }
    
    @Override
    public <A extends Annotation> A getAnnotation(Class<A> obj) {
        Objects.requireNonNull(obj);
        return (A)this.annotationData().annotations.get(obj);
    }
    
    @Override
    public boolean isAnnotationPresent(Class<? extends Annotation> annotationClass) {
        return super.isAnnotationPresent(annotationClass);
    }
    
    @Override
    public <A extends Annotation> A[] getAnnotationsByType(Class<A> clazz) {
        Objects.requireNonNull(clazz);
        return AnnotationSupport.getAssociatedAnnotations(this.annotationData().declaredAnnotations, this, clazz);
    }
    
    @Override
    public Annotation[] getAnnotations() {
        return AnnotationParser.toArray(this.annotationData().annotations);
    }
    
    @Override
    public <A extends Annotation> A getDeclaredAnnotation(Class<A> obj) {
        Objects.requireNonNull(obj);
        return (A)this.annotationData().declaredAnnotations.get(obj);
    }
    
    @Override
    public <A extends Annotation> A[] getDeclaredAnnotationsByType(Class<A> clazz) {
        Objects.requireNonNull(clazz);
        return AnnotationSupport.getDirectlyAndIndirectlyPresent(this.annotationData().declaredAnnotations, clazz);
    }
    
    @Override
    public Annotation[] getDeclaredAnnotations() {
        return AnnotationParser.toArray(this.annotationData().declaredAnnotations);
    }
    
    private Class.AnnotationData annotationData() {
        while (true) {
            Class.AnnotationData annotationData = this.annotationData;
            int classRedefinedCount = this.classRedefinedCount;
            if (annotationData != null && annotationData.redefinedCount == classRedefinedCount) {
                return annotationData;
            }
            Class.AnnotationData annotationData2 = this.createAnnotationData(classRedefinedCount);
            if (Class.Atomic.casAnnotationData(this, annotationData, annotationData2)) {
                return annotationData2;
            }
            continue;
        }
    }
    
    private Class.AnnotationData createAnnotationData(int n) {
        Map<Class, Object> annotations = AnnotationParser.parseAnnotations(this.getRawAnnotations(), this.getConstantPool(), this);
        Class<Object> superclass = this.getSuperclass();
        annotations = null;
        if (superclass != null) {
            Map<Object, Object> annotations2 = superclass.annotationData().annotations;
            for (Map.Entry entry : annotations2.entrySet()) {
                Class<Object> annotationClass = (Class)entry.getKey();
                if (AnnotationType.getInstance(annotationClass).isInherited()) {
                    if (annotations == null) {
                        annotations = new LinkedHashMap((Math.max(annotations.size(), Math.min(12, annotations.size() + annotations2.size())) * 4 + 2) / 3);
                    }
                    annotations.put(annotationClass, entry.getValue());
                }
                else {
                    continue;
                }
            }
        }
        if (annotations == null) {
        }
        else {
            annotations.putAll(annotations);
        }
        return new Class.AnnotationData(annotations, annotations, n);
    }
    
    boolean casAnnotationType(AnnotationType annotationType, AnnotationType annotationType2) {
        return Class.Atomic.casAnnotationType(this, annotationType, annotationType2);
    }
    
    AnnotationType getAnnotationType() {
        return this.annotationType;
    }
    
    Map<Class<? extends Annotation>, Annotation> getDeclaredAnnotationMap() {
        return this.annotationData().declaredAnnotations;
    }
    
    public AnnotatedType getAnnotatedSuperclass() {
        if (this == Object.class || this.isInterface() || this.isArray() || this.isPrimitive() || this == Void.TYPE) {
            return null;
        }
        return TypeAnnotationParser.buildAnnotatedSuperclass(this.getRawTypeAnnotations(), this.getConstantPool(), this);
    }
    
    public AnnotatedType[] getAnnotatedInterfaces() {
        return TypeAnnotationParser.buildAnnotatedInterfaces(this.getRawTypeAnnotations(), this.getConstantPool(), this);
    }
    
    static Field[] access$100(Class clazz, boolean b) {
        return clazz.getDeclaredFields0(b);
    }
    
    static Field access$200(Field[] array, String s) {
        return searchFields(array, s);
    }
    
    static boolean access$300(Object[] array, Object[] array2) {
        return arrayContentsEq(array, array2);
    }
    
    static {
        registerNatives();
        Class.useCaches = true;
        serialPersistentFields = new ObjectStreamField[0];
        Class.initted = false;
    }
    
    private static final class EnclosingMethodInfo
    {
        private Class<?> enclosingClass;
        private String name;
        private String descriptor;
        
        private EnclosingMethodInfo(Object[] array) {
            if (array.length != 3) {
                throw new InternalError("Malformed enclosing method information");
            }
            try {
                this.enclosingClass = (Class)array[0];
                assert this.enclosingClass != null;
                this.name = (String)array[1];
                this.descriptor = (String)array[2];
                assert this.name == this.descriptor;
            }
            catch (ClassCastException ex) {
                throw new InternalError("Invalid type in enclosing method information", ex);
            }
            return;
        }
        
        boolean isPartial() {
            return this.enclosingClass == null || this.name == null || this.descriptor == null;
        }
        
        boolean isConstructor() {
            return !this.isPartial() && "<init>".equals(this.name);
        }
        
        boolean isMethod() {
            return !this.isPartial() && !this.isConstructor() && !"<clinit>".equals(this.name);
        }
        
        Class<?> getEnclosingClass() {
            return this.enclosingClass;
        }
        
        String getName() {
            return this.name;
        }
        
        String getDescriptor() {
            return this.descriptor;
        }
    }
    
    private static class Atomic
    {
        private static final Unsafe unsafe;
        private static final long reflectionDataOffset;
        private static final long annotationTypeOffset;
        private static final long annotationDataOffset;
        
        private static long objectFieldOffset(Field[] array, String s) {
            Field access$200 = Class.access$200(array, s);
            if (access$200 == null) {
                throw new Error("No " + s + " field found in java.lang.Class");
            }
            return Class.Atomic.unsafe.objectFieldOffset(access$200);
        }
        
        static <T> boolean casReflectionData(Class<?> o, SoftReference<Class.ReflectionData<T>> expected, SoftReference<Class.ReflectionData<T>> x) {
            return Class.Atomic.unsafe.compareAndSwapObject(o, Class.Atomic.reflectionDataOffset, expected, x);
        }
        
        static <T> boolean casAnnotationType(Class<?> o, AnnotationType expected, AnnotationType x) {
            return Class.Atomic.unsafe.compareAndSwapObject(o, Class.Atomic.annotationTypeOffset, expected, x);
        }
        
        static <T> boolean casAnnotationData(Class<?> o, Class.AnnotationData expected, Class.AnnotationData x) {
            return Class.Atomic.unsafe.compareAndSwapObject(o, Class.Atomic.annotationDataOffset, expected, x);
        }
        
        static {
            unsafe = Unsafe.getUnsafe();
            Field[] access$100 = Class.access$100(Class.class, false);
            reflectionDataOffset = objectFieldOffset(access$100, "reflectionData");
            annotationTypeOffset = objectFieldOffset(access$100, "annotationType");
            annotationDataOffset = objectFieldOffset(access$100, "annotationData");
        }
    }
    
    private static class ReflectionData<T>
    {
        volatile Field[] declaredFields;
        volatile Field[] publicFields;
        volatile Method[] declaredMethods;
        volatile Method[] publicMethods;
        volatile Constructor<T>[] declaredConstructors;
        volatile Constructor<T>[] publicConstructors;
        volatile Field[] declaredPublicFields;
        volatile Method[] declaredPublicMethods;
        volatile Class<?>[] interfaces;
        final int redefinedCount;
        
        ReflectionData(int redefinedCount) {
            this.redefinedCount = redefinedCount;
        }
    }
    
    static class MethodArray
    {
        private Method[] methods;
        private int length;
        private int defaults;
        
        MethodArray() {
            this(20);
        }
        
        MethodArray(int n) {
            if (n < 2) {
                throw new IllegalArgumentException("Size should be 2 or more");
            }
            this.methods = new Method[n];
            this.length = 0;
            this.defaults = 0;
            return;
        }
        
        boolean hasDefaults() {
            return this.defaults != 0;
        }
        
        void add(Method method) {
            if (this.length == this.methods.length) {
                this.methods = (Method[])Arrays.copyOf(this.methods, 2 * this.methods.length);
            }
            this.methods[this.length++] = method;
            if (method != null && method.isDefault()) {
                ++this.defaults;
            }
        }
        
        void addAll(Method[] array) {
            for (int i = 0; i < array.length; ++i) {
                this.add(array[i]);
            }
        }
        
        void addAll(Class.MethodArray methodArray) {
            for (int i = 0; i < methodArray.length(); ++i) {
                this.add(methodArray.get(i));
            }
        }
        
        void addIfNotPresent(Method obj) {
            int i = 0;
            while (i < this.length) {
                Method method = this.methods[i];
                if (method == obj || (method != null && method.equals(obj))) {
                    return;
                }
                ++i;
            }
            this.add(obj);
        }
        
        void addAllIfNotPresent(Class.MethodArray methodArray) {
            for (int i = 0; i < methodArray.length(); ++i) {
                Method value = methodArray.get(i);
                if (value != null) {
                    this.addIfNotPresent(value);
                }
            }
        }
        
        void addInterfaceMethods(Method[] array) {
            for (Method method : array) {
                if (!Modifier.isStatic(method.getModifiers())) {
                    this.add(method);
                }
            }
        }
        
        int length() {
            return this.length;
        }
        
        Method get(int n) {
            return this.methods[n];
        }
        
        Method getFirst() {
            Method[] methods = this.methods;
            int length = methods.length;
            int i = 0;
            while (i < length) {
                Method method = methods[i];
                if (method != null) {
                    return method;
                }
                ++i;
            }
            return null;
        }
        
        void removeByNameAndDescriptor(Method method) {
            for (int i = 0; i < this.length; ++i) {
                Method method2 = this.methods[i];
                if (method2 != null && this.matchesNameAndDescriptor(method2, method)) {
                    this.remove(i);
                }
            }
        }
        
        private void remove(int n) {
            if (this.methods[n] != null && this.methods[n].isDefault()) {
                --this.defaults;
            }
            this.methods[n] = null;
        }
        
        private boolean matchesNameAndDescriptor(Method method, Method method2) {
            return method.getReturnType() == method2.getReturnType() && method.getName() == method2.getName() && Class.access$300(method.getParameterTypes(), method2.getParameterTypes());
        }
        
        void compactAndTrim() {
            int newLength = 0;
            for (int i = 0; i < this.length; ++i) {
                Method method = this.methods[i];
                if (method != null) {
                    if (i != newLength) {
                        this.methods[newLength] = method;
                    }
                    ++newLength;
                }
            }
            if (newLength != this.methods.length) {
                this.methods = (Method[])Arrays.copyOf(this.methods, newLength);
            }
        }
        
        void removeLessSpecifics() {
            if (!this.hasDefaults()) {
                return;
            }
            for (int i = 0; i < this.length; ++i) {
                Method value = this.get(i);
                if (value != null && value.isDefault()) {
                    for (int j = 0; j < this.length; ++j) {
                        if (i != j) {
                            Method value2 = this.get(j);
                            if (value2 != null && this.matchesNameAndDescriptor(value, value2) && hasMoreSpecificClass(value, value2)) {
                                this.remove(j);
                            }
                        }
                    }
                }
            }
            return;
        }
        
        Method[] getArray() {
            return this.methods;
        }
        
        static boolean hasMoreSpecificClass(Method method, Method method2) {
            Class<Object> declaringClass = method.getDeclaringClass();
            Class<Object> declaringClass2 = method2.getDeclaringClass();
            return declaringClass != declaringClass2 && declaringClass2.isAssignableFrom(declaringClass);
        }
    }
    
    private static class AnnotationData
    {
        final Map<Class<? extends Annotation>, Annotation> annotations;
        final Map<Class<? extends Annotation>, Annotation> declaredAnnotations;
        final int redefinedCount;
        
        AnnotationData(Map<Class<? extends Annotation>, Annotation> annotations, Map<Class<? extends Annotation>, Annotation> declaredAnnotations, int redefinedCount) {
            this.annotations = annotations;
            this.declaredAnnotations = declaredAnnotations;
            this.redefinedCount = redefinedCount;
        }
    }
}
